class DotEnvError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class NytApiError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class ConfigNotFoundError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class NLPNotFoundError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


import numpy as np

from .interest import get_interest, get_mid
from .anomalies import constant_sd, rolling_std, ewm_std
from .visualize import plot_data_plotly, plot_data, plot_data_with_anomalies, plot_data_with_anomalies_plotly
from .articles import get_articles_title_text_images_all_dates
from .LDA import run_lda
from .summarizer import lda_filter_articles_anomalies, get_summaries_by_topic, run_summary

class Huginn:
    def __init__(self, keyword, mid=True):
        """Create a Huginn object
        :argument keyword: person or entity you would like information about
        """
        self.name = keyword
        if mid: self.__mid = get_mid(self.name) #private attribute
        else: self.__mid = None
        self.interest = get_interest(self.name, self.__mid)

    def get_anomalies(self, method="ewm", **kwargs):
        """Get anomalies under method assumption (by default ewm)

        :argument method: ewm, rolling or constant
        :argument **kwargs: if ewm halflife_mean, halflife_std, k (set to 1, 10 and 1 by default)
                            if rolling lookback_mean, lookback_std, k (set to 1, 10 and 1 by default)
                            if constant k (set to 1 by default)

        :returns the anomalies as a DateIndex
        """
        if method == "ewm":
            self.anomalies = ewm_std(self.interest, **kwargs)
        if method == "rolling":
            self.anomalies = rolling_std(self.interest, **kwargs)
        if method == "constant":
            self.anomalies = constant_sd(self.interest, **kwargs)
        self.anomalies_formatted = np.array(self.anomalies, dtype='datetime64[D]')
        return self.anomalies

    #private method
    def __check_got_anomalies(self):
        """Method to check if get_anomalies has been called, used primarily as a check in later functions"""
        if not hasattr(self, 'anomalies'):
            raise AttributeError('Huginn has not gotten anomalies yet. Use \'get_anomalies\'.')

    #private method
    def __check_got_articles(self):
        """Method to check if get_articles_info has been called, used primarily as a check in later functions"""
        if not hasattr(self, 'articles'):
            raise AttributeError('Huginn has not gotten articles yet. Use \'get_articles_info\'.')

    #private method
    def __check_got_summaries(self):
        """Method to check if get_articles_info_and_summary_after_LDA has been called, used primarily as a check in later functions"""
        if not hasattr(self, 'summary_by_anomalies_by_topics'):
            raise AttributeError('Huginn has not gotten summaries yet. Use \'get_articles_info_and_summary_after_LDA\'.')

    def plot_interest(self, plotly=False):
        """Plot only the interest the month of the entity or person under study"""
        if not plotly:
            plot_data(self.interest)
        else:
            return plot_data_plotly(self.interest)

    def plot_interest_with_anomalies(self, plotly=False):
        """
        Plot interest by month and the anomalies (as vertical lines)
        """
        self.__check_got_anomalies()
        if not plotly:
            plot_data_with_anomalies(self.interest, self.anomalies)
        else:
            return plot_data_with_anomalies_plotly(self.interest, self.anomalies)

    def get_articles_info(self, num_links='all'):
        """Get all information about articles (images, urls, content, titles) for each anomaly
        :argument num_links: number of links to keep for each anomaly ('all' by default). The maximum number of articles is 10 due to API quota limit.
        :returns dictionary, keys are anomaly dates and values are list of images, urls, contents or titles
        """
        self.__check_got_anomalies() #check if we have anomalies
        tmp = get_articles_title_text_images_all_dates(self.name, self.anomalies, num_links)
        self.urls = tmp['urls']
        self.titles = tmp['titles']
        self.articles = tmp['texts']
        self.images = tmp['images']

    #private method
    def __get_topics_with_lda(self, n_components):
        """Must have run get_anomalies() and get_title_text() to have requisite articles in session
           prior to running LDA on the object
        Get distribution of articles through topics for each anomaly date
        :argument n_components: number of topics for LDA, set to 2 by default (out of scope and in focus area).
        :returns a dictionary, keys are dates, values are dictionary (keys are topics and values are list of articles)
        """
        self.__check_got_anomalies()
        self.__check_got_articles()
        self.__lda_output = run_lda(self.articles, n_components=n_components) #private attributes

    def get_local_summaries(self, n_components = 2, min_length = 50, max_length = 150):
        """Compute the summary for each anoamly date
        :argument max_length: int, max length of the summary
        :argument n_components: number of topics for LDA
        :returns a summary (str) for each anomaly date, for each topic (dic of dic)
        """
        self.__check_got_anomalies()
        self.__check_got_articles()
        self.__get_topics_with_lda(n_components = n_components)

        lda_filter_articles = lda_filter_articles_anomalies(self.__lda_output, self.articles)
        self.summary_by_anomalies_by_topics = get_summaries_by_topic(lda_filter_articles, min_length, max_length)

        return self.summary_by_anomalies_by_topics

    def get_global_summary(self, min_length = 50, max_length = 150):
        """Get a global summary (summarize all articles as one)
        No LDA, no matter what the anomaly date is.
        """
        self.__check_got_anomalies()
        self.__check_got_articles()

        sentence = ' '.join([text for texts in list(self.articles.values()) for text in texts]) #concatenate all articles

        self.global_summary = run_summary(sentence, min_length, max_length)
        return self.global_summary

    def get_summary_of_summaries(self, min_length = 50, max_length = 150):
        """Get summary of summaries to compare with get_global_summary
        :argument max_length: max length of the summary
        :returns str (summary)
        """
        self.__check_got_summaries() #check if summaries are available

        tmp = [list(dic.values()) for dic in list(self.summary_by_anomalies_by_topics.values())]
        sentence = ' '.join([text for texts in tmp for text in texts]) #concatenate all summaries
        self.summary_of_summaries = run_summary(sentence, min_length, max_length)
        return self.summary_of_summaries



##################################
# GET INTEREST FILE (volume of Google Searches per month)

from pytrends.request import TrendReq
import pandas as pd
import os

def get_pytrend():
    """Get pytrends to scrap Google Trends data"""
    pytrends = TrendReq(hl='en-US', #language
                        tz=360) #timezone (US CST is 360)
    return pytrends

def get_mid(keyword):
    """Ask for the mid your are interested in. Pytrends has a very useful method '.suggestions' which enables to
    specify your research (Ex: Apple could be a company or a fruit)
    """
    pytrends = get_pytrend()
    print(pytrends.suggestions(keyword))
    mid = input('Enter the mid you are interested in: ')
    return mid

def get_interest(keyword, mid=None):
    """Get the time series of interest by month for the selected entity (keyword and mid)

    :argument keyword: entity name
    :argument mid: returned by the above function to get precision

    :return: dataframe with dates as index and one column named keyword which corresponds to the interest by month
    """
    try:
        interest = pd.read_csv(os.path.join(os.path.dirname(__file__), 'interest_cache/' + keyword + '.csv'), index_col=0)
        interest.index = pd.to_datetime(interest.index)
        return interest
    except:
        pytrends = get_pytrend()

        if not mid:
            mid = keyword

        pytrends.build_payload([mid], #up to 5 terms in the list
                               cat=0, #default to no category
                               timeframe='all', #Date to start from
                               geo='', #two letter country abreviation (default to world)
                               gprop='news') #what property to filter to (images, news, youtube or froogle
                                         #(for Google Shopping results))
        data = pytrends.interest_over_time() #two columns (mid name <-> interest and isPartial) + index = date
        data.drop(columns = 'isPartial', inplace = True) #rermove isPartial column
        data.columns = [keyword] #rename the column with the keyword name instead of the mid name
        return data



##################################
# LDA

import spacy, gensim
import pandas as pd
import numpy as np
from sklearn.decomposition import LatentDirichletAllocation
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.decomposition import LatentDirichletAllocation

from .exceptions import NLPNotFoundError

def sent_to_words(sentences):
    """From list of sentence (str) to a list of list of words (str)
    :argument sentences: list of str (list of sentences)
    :returns a generator of list of words (each word of the sentence)
    """
    for sentence in sentences:
        yield(gensim.utils.simple_preprocess(str(sentence), deacc=True))  # deacc=True removes punctuations

def lemmatization(texts, nlp, allowed_postags=['NOUN', 'ADJ', 'VERB', 'ADV']):
    """Lemmatize the text
    :argument texts: list of list of words
    :argument nlp: spacy model (english in our case)
    :argument allowed_postags: list (type of words to keep)
    :returns a list of lemmatized sentences
    """
    texts_out = []
    for sent in texts:
        doc = nlp(" ".join(sent)) 
        texts_out.append(" ".join([token.lemma_ if token.lemma_ not in ['-PRON-'] else '' 
                                   for token in doc if token.pos_ in allowed_postags]))
    return texts_out

def vectorize(vectorizer, sentences):
    """Vectorizes sentences
    :argument vectorize: CountVectorize
    :argument sentences: list of str (list of sentence)
    :returns a sparse matrix to feed LDA with (for each document, number of each word) (LDA doesn't take order into account)
    """
    return vectorizer.fit_transform(sentences)

def preprocess(sentences, nlp):
    """Preprocess sentences to feed LDA algorithm (sparse matrix speeds up the training)
    :argument sentences: list of str (sentences)
    :argument nlp: spacy language model
    :returns a sparse matrix and the vectorizer
    """
    #for each anomaly date, we need a vectorizer
    vectorizer = CountVectorizer(analyzer='word',               
                                 stop_words='english',             # remove stop words
                                 lowercase=True,                   # convert all words to lowercase
                                 token_pattern='[a-zA-Z0-9]{3,}')  # num chars > 3
    
    data = list(sent_to_words(sentences)) #remove ponctuation and convert to list of list
    data = lemmatization(data, nlp) #remove pronouns and other words, convert to list of sentences
    data = vectorize(vectorizer, sentences) #remove stopwords, lowercase, num_chars > 3
    return data, vectorizer

def run_lda_once(sentences, nlp, n_components):
    """Run LDA algorithm for only one anomaly date
    :argument n_components: int (number of topics, set to 2 by default: out of scope and in the scope)
    :argument sentences: list of str (sentences)
    :argument nlp: spacy language model
    :returns a dictionary: keys are topics and values are ids of articles to keep for each topic (for each topic t, only keep articles whose dominant topic is t)
    """
    #preprocess sentences
    preprocess_data, vectorizer = preprocess(sentences, nlp)
    
    lda = LatentDirichletAllocation(n_components = n_components, random_state=230, max_iter=20)
    lda.fit(preprocess_data) #train LDA
    #return partition of each topic for each document (dense array of size n_doc * n_components)
    lda_output = lda.transform(preprocess_data) 

    # Create dataframe
    topicnames = ["Topic" + str(i) for i in range(n_components)] #column names
    docnames = ["Doc" + str(i) for i in range(preprocess_data.shape[0])] # index names
    df_document_topic = pd.DataFrame(lda_output, columns=topicnames, index=docnames)

    # Get dominant topic for each document
    dominant_topic = np.argmax(df_document_topic.values, axis=1)
    df_document_topic['dominant_topic'] = dominant_topic
    
    ids_topic = {} #create number of topics set of articles (this is a partition of the set of all articles)
    topics = list(df_document_topic.groupby('dominant_topic').count()['Topic0'].sort_values(ascending = False).index) #sort topics by dominance across documents
    for topic in topics:
        ids_topic[topic] = list(map(lambda x: int(x[3:]),
                          list(df_document_topic[df_document_topic['dominant_topic']==topic].index)))
    
    return ids_topic

def run_lda(dic_sentences, n_components):
    """Run LDA algorithm
    :argument dic_sentences: dic, keys are date and values are list of sentences
    :argument n_components: int (number of topics, set to 2 by default: out of scope and in the scope)
    :returns a dic - keys are dates, values are a list of 3 dataframes (see run_lda_once for more information)
    """
    # Initialize spacy 'en' model, keeping only tagger component (for efficiency)
    try:
        nlp = spacy.load('en', disable=['parser', 'ner'])
    except:
        raise NLPNotFoundError("No spacy 'en' model were found, please run in terminal: python3 -m spacy download en")
        
    dates = dic_sentences.keys()
    dic_info = {}
    for date in dates:
        dic_info[date] = run_lda_once(dic_sentences[date], nlp, n_components=n_components)
    return dic_info


##################################
# FUNCTIONS TO GET ARTICLE URLs, TEXTS, TITLES

import requests
import os
from pathlib import Path

import pandas as pd
from bs4 import BeautifulSoup

from .exceptions import NytApiError, DotEnvError, ConfigNotFoundError

def _timestamp_to_string(timestamp):
    """Extracts the date from a pandas timestamp and convert to a string

    :returns string of the format: '20190908' (for September 8th, 2019)

    """
    date_dashes = str(timestamp.date()).replace('-', '')
    return date_dashes

def _get_sections():
    """ Get the desired sections to search from a config file"""
    file_path = Path(os.path.dirname(os.path.abspath(__file__))) / 'config' / 'sections.txt'
    try:
        with open(file_path, 'r') as f:
            sections = f.readlines()

    except FileNotFoundError:
        raise ConfigNotFoundError('Config file not found')

    section_list = ["\"{}\"".format(line.strip()) for line in sections]
    section_string = " ".join(section_list)

    return section_string

def get_api_key():
    """Uses .env files to fetch and return the NYT API Key"""
    api_key = os.environ.get('NYT_API_KEY')

    if api_key is None:
        raise DotEnvError('No \'NYT_API_KEY\' system environment variable defined')

    return api_key

def get_nyt_url(keyword, date):
    """Construct NYT API URL

    :argument keyword: entity you are interested in
    :argument date: pandas timestemp

    :returns: corresponding url as a string
    """
    begin_date = _timestamp_to_string(date - pd.DateOffset(days=2))
    end_date = _timestamp_to_string(date + pd.DateOffset(months = 1) + pd.DateOffset(days=2))
    start_url = 'https://api.nytimes.com/svc/search/v2/articlesearch.json?'
    api_key = get_api_key()
    sections = _get_sections()

    url = (start_url+"q={}&fq=section_name:({})&api-key={}&begin_date={}&end_date={}") \
        .format(keyword, sections, api_key, begin_date, end_date)
    return url

def get_article_urls(keyword, date, num_links='all'):
    """ Returns the links to NYT in the month preceding the input date with a keyword input

    Note - we search the month before the anomaly date as well

    :argument keyword: str keyword to search by
    :argument date: pd.Timestamp containing the date of anomaly, to search the month before
    :argument num_links: The number of links to return. Default is all.

    :returns a list of links of length num_links (corresponding to articles for ONE anomaly)
    """
    url = get_nyt_url(keyword, date)
    r = requests.get(url)

    # Look for a fault in the returned data
    try:
        fault = r.json()['fault']['faultstring']
        raise NytApiError(fault)

    except KeyError:
        #only keep article (not multimedia content)
        articles = [news['web_url'] for news in r.json()['response']['docs'] if news['document_type'] == 'article']
        if num_links == 'all':
            return articles
        else:
            return articles[:num_links]

def get_article_title_text_images(article_url):
    """Get the images, title and text of ONE article from its url

    :argument article_url: str corresponding to the web url of the article

    :returns the images, title and content of the article as a string
    """
    r = requests.get(article_url)
    soup = BeautifulSoup(r.content, features="lxml")
    try:
        image_links = []
        for link in soup.find_all('img'):
            imgSrc = link.get('src')
            image_links.append(imgSrc)

        article_html_content = soup.find('html').find('body').find('article')

        title = article_html_content.find('header').find('h1').get_text()
        section = article_html_content.find('section')
        div = article_html_content.find('div', {"class": "entry-content"})

        if section: #NYT article
            paragraphs = section.find_all('p')
        else: #NYT blog article
            paragraphs = div.find_all('p')

        text = ' '.join([title+'.'] + [p.get_text() for p in paragraphs])
        return image_links, title, text
    except:
        return None, None, None

def get_articles_title_text_images(keyword, date, num_links = 'all'):
    """Get the images, title and text for ALL articles related to keyword at a specific date

    :argument keyword: str keyword (entity)
    :argument date: pandas datetime (anomaly date)
    :argument num_links: number of links (articles) to consider, default to all

    :returns a dictionary (keys are urls, images, titles and texts), values are a list of str, corresponding to the urls, images, title or text of each article and an int (S), the number of articles that could have been scrapped
    """
    articles_url = get_article_urls(keyword, date, num_links=num_links)
    results = {}
    results['urls'], results['titles'], results['texts'], results['images'] = articles_url, [], [], []
    S = 0
    for url in articles_url:
        images, title, text = get_article_title_text_images(url)
        if images is not None:
            S+=1
            results['titles'].append(title)
            results['texts'].append(text)
            results['images'].append(images)
    return S, results

def get_articles_title_text_images_all_dates(keyword, dates, num_links = 'all'):
    """Get ALL articles urls, images, title and text for ALL dates (anomalies) related to keyword (entity or person)

    :argument keyword: str keyword (entity or person)
    :argument dates: DatetimeIndex of pandas datetime (dtype=datetime64[ns])
    :argument num_links: number of links (articles) to consider, default to all

    :returns a dictionary (keys are titles and texts) of dictionary whose keys are dates (anomalies) and values are lists containing urls, titles, images or text of articles
    """
    results = {'urls': {}, 'titles':{}, 'texts':{}, 'images':{}}
    for i,date in enumerate(dates):
        S, tmp = get_articles_title_text_images(keyword, date, num_links)
        results['urls'][date] = tmp['urls']
        results['titles'][date] = tmp['titles']
        results['texts'][date] = tmp['texts']
        results['images'][date] = tmp['images']
        N=len(tmp['urls'])
        print('anomaly n°{0}: {1} articles were found and {2}% were retrieved'.format(
            str(i+1), str(N), str(int(S/N*100))))
    return results


##################################
# ANOMALIES

def constant_sd(data, k = 1):
    """method to get anomalies as dates (DatetimeIndex) (dtype=datetime64[ns])
    - constant mean mu
    - constand std sigma
    - data points xn
    - anomalies definition: xn - mu > k*std

    - maximum ten anomalies (10 'biggest' anomalies)

    :argument data: dataframe with one column (interest) and pandas dates as index
    :argument k: float

    :returns anomalies as DatetimeIndex sorted(dtype=datetime64[ns])
    """
    mean = data.mean()[0]
    std = data.std()[0]
    tmp = data-mean-k*std
    return tmp[tmp.iloc[:,0]>0].sort_values(by = tmp.columns[0], ascending=False)[0:10].index.sort_values()

def rolling_std(data, lookback_mean = 1, lookback_std = 10, k = 1):
    """method to get anomalies as dates (DatetimeIndex) (dtype=datetime64[ns])
    - rolling mean mu_hat(n)
    - rolling std sigma_hat(n)
    - data points xn
    - anomalies definition: xn - mu_hat(n-1) > k*std_hat(n-1)

    - maximum ten anomalies (10 'biggest' anomalies)

    :argument data: dataframe with one column (interest) and pandas dates as index
    :argument k: float

    :returns anomalies as DatetimeIndex sorted(dtype=datetime64[ns])
    """
    mean = data.rolling(lookback_mean).mean().shift(1)
    std = data.rolling(lookback_std).std().shift(1)
    std = std[std>0]
    tmp = (data-mean)/(k*std)
    return tmp[tmp.iloc[:,0]>1].sort_values(by = tmp.columns[0], ascending = False)[0:10].index.sort_values()

def ewm_std(data, halflife_mean=1, halflife_std=10, k = 1): 
    """method to get anomalies as dates (DatetimeIndex) (dtype=datetime64[ns])
    - exponential moving weighted (emw) mean mu_hat(n)
    - emw std sigma_hat(n)
    - data points xn
    - anomalies definition: xn - mu_hat(n-1) > k*std_hat(n-1)

    - maximum ten anomalies (10 'biggest' anomalies)

    :argument data: dataframe with one column (interest) and pandas dates as index
    :argument k: float

    :returns anomalies as DatetimeIndex sorted(dtype=datetime64[ns])
    """
    mean = data.ewm(halflife = halflife_mean).mean().shift(1)
    std = data.ewm(halflife = halflife_std).std().shift(1)
    std = std[std>0]
    tmp = (data-mean)/(k*std)
    return tmp[tmp.iloc[:,0]>1].sort_values(by = tmp.columns[0], ascending = False)[0:10].index.sort_values()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='huginn',
      version='0.1.4',
      description='Tools to Detect Anomalous Events and News Related to Entities',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://github.com/tcausero/huginn',
      author='Jesse Cahill, Thomas Causero, James DeAntonis, Ryan McNally',
      author_email='jcahill225@gmail.com, tc3030@columbia.edu, jad2295@columbia.edu, rom2109@columbia.edu',
      license='MIT',
      packages=find_packages(exclude=('tests',)),
      include_package_data=True,
      python_requires='>=3.5, <3.7',
      install_requires=[
            'pandas',
            'matplotlib',
            'pytrends',
            'geopandas',
            'scikit-learn',
            'colour',
            'numpy',
            'click',
            'bs4',
            'plotly',
            'pyLDAvis',
            'gensim',
            'tokenizer',
            'spacy',
            'nltk',
            'torch',
            'transformers'
       ],
      zip_safe=False)



##################################
# SUMMARIZER

from transformers import pipeline

def lda_filter_articles(ids, articles):
    """From sets of ids to set of sentences (str)
    :argument ids: dictionary (keys are topics and values are ids of articles
    :argument articles: list of str
    :returns a dictionary, keys are topics and values are sentences (str)
    """
    articles_per_topic = {}
    for topic in ids.keys():
        tmp_topic_articles = [article for i,article in enumerate(articles) if i in ids[topic]]
        articles_per_topic[topic] = ' '.join(tmp_topic_articles)
    return articles_per_topic

def lda_filter_articles_anomalies(dic_ids, dic_articles):
    """Preprocess articles for each anomaly date (with LDA output)
    :argument dic_ids: dictionary of dictionary, keys are dates and values are dictionary whose keys are topics and values are list of int (ids)
    :argument dic_articles: dictionary, keys are dates and values are list of articles
    :returns preprocess articles, dictionary of dictionary, keys are dates and values are dictionary whose keys are topics and values are sentences (str)
    """
    dates = dic_articles.keys()
    preprocessed_LDA_articles = {}
    for date in dates:
        preprocessed_LDA_articles[date] = lda_filter_articles(dic_ids[date], dic_articles[date])
    return preprocessed_LDA_articles

def run_summary(sentence, min_length, max_length):
    """Summarize sentence with a maximum of max_length characters
    :argument sentence: str (sentence to summarize)
    :argument max_length: maximum length of the output
    :returns str: summary of the sentence
    """
    summarizer = pipeline("summarization")
    return summarizer(sentence, min_length=min_length, max_length=max_length)[0]['summary_text']

def get_summaries_by_topic(dic_anoamlies_topic_articles, min_length, max_length):
    """Compute the summary for each anoamly date
    Articles are supposed to be already filtered by LDA
    :argument dic_anoamlies_topic_articles: dic, keys are dates, values are dic (keys are topics, values are sentences (str))
    :argument max_length: int, max length of the summary
    :returns a summary (str) for each anomaly date, for each topic (dic of dic)
    """
    summaries_by_topic = {}
    dates = dic_anoamlies_topic_articles.keys()
    for date in dates:
        summaries_by_topic[date.strftime('%m-%Y')] = {}
        for topic in dic_anoamlies_topic_articles[date].keys():
            summaries_by_topic[date.strftime('%m-%Y')][topic] = run_summary(dic_anoamlies_topic_articles[date][topic], 
                                                              min_length, max_length)
    return summaries_by_topic


##################################
# PLOT DATA

import matplotlib.pyplot as plt
import pandas as pd
import plotly
import plotly.express as px

def plot_data_plotly(data):
    fig = px.line(data, x=data.index, y=data.columns[0])

    fig.update_layout(
        title='Google Search Interest in '+ data.columns[0] + ' over Time',
        xaxis_title="Year",
        yaxis_title="Interest (Standardized)",
        font=dict(
            family="Courier New, monospace",
            size=12,
            color="#7f7f7f"
            )
        )
    return plotly.offline.plot(fig, auto_open=False, output_type='div')

def plot_data(data):
    """ Plot the data (time series)

    :argument data: dataframe with time as index and interests (volume of google searches) as first column
    Note that the name of the first column is the name of the enity
    """
    fig, ax = plt.subplots(1, 1, figsize=(20, 5))
    ax.plot(data, label = data.columns[0])
    ax.set_xlabel('Year')
    ax.set_ylabel('Interest (Standardized)')
    ax.set_title('Google Search Interest in '+ data.columns[0] + ' over Time')
    fig.autofmt_xdate()
    plt.legend()
    plt.show()

def plot_data_with_anomalies_plotly(data, anomalies):
    fig = px.line(data, x=data.index, y=data.columns[0])

    shapes = []
    for anomaly in anomalies:
        shapes.append({# Unbounded span at 6 <= x <= 8
                'type': 'rect',
                # x-reference is assigned to the x-values
                'xref': 'x',
                # y-reference is assigned to the plot paper [0,1]
                'yref': 'paper',
                'x0': anomaly - pd.DateOffset(15),
                'y0': 0,
                'x1': anomaly + pd.DateOffset(15),
                'y1': 1,
                'fillcolor': 'red',
                'opacity': 0.2,
                'line': {
                    'width': 0,
                }
            }
        )

    fig.update_layout(
        title='Google Search Interest in '+ data.columns[0] + ' over Time',
        xaxis_title="Year",
        yaxis_title="Interest (Standardized)",
        font=dict(
            family="Courier New, monospace",
            size=12,
            color="#7f7f7f"
            ),
        shapes=shapes
        )
    return plotly.offline.plot(fig, auto_open=False, output_type='div')

def plot_data_with_anomalies(data, anomalies):
    """ Plot the data and add anomalies as line on the graph

    :argument data: dataframe (same as above)
    :argument anomalies: DatetimeIndex of dates (corresponding to anomalies) - dtype=datetime64[ns]
    """
    fig, ax = plt.subplots(1, 1, figsize=(20, 5))
    ax.plot(data, label = data.columns[0])
    ax.set_xlabel('Year')
    ax.set_ylabel('Interest (Standardized)')
    ax.set_title('Google Search Interest in '+ data.columns[0] + ' over Time')
    fig.autofmt_xdate()

    for anomaly in anomalies:
        ax.axvspan(anomaly - pd.DateOffset(5),
                     anomaly + pd.DateOffset(5),
                     color='y', alpha=0.5, lw=0)

    plt.legend()
    plt.show()
    print("""
    If you are not happy with these anomalies, you can call the method \'get_anomalies\' and
    specify the function to get anomalies:
    - method = 'constant' with parameter k (set to 1 by default)
    - method = 'rolling' with parameters lookback_mean, lookback_std and k (set to 1, 10, 1 by default)
    - method = 'ewm' with parameters halflife_mean, halflife_std and k(set to 1,10,1 by default) [default method]
    """)